SystemOrganization addCategory: #MCInstaller!

Object subclass: #MczInstaller
	instanceVariableNames: 'zip '
	classVariableNames: 'Versions '
	poolDictionaries: ''
	category: 'MCInstaller'!

!MczInstaller class methodsFor: 'versionInfo' stamp: 'cwp 8/11/2003 23:49'!
storeVersionInfo: aVersion
	Versions 
		at: aVersion package name
		put: aVersion info asDictionary! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'bf 2/9/2004 15:00'!
versions: aVersionList anySatisfy: aDependencyID
	^ aVersionList anySatisfy: [:version | 
			aDependencyID = (version at: #id)
				or: [self versions: (version at: #ancestors) anySatisfy: aDependencyID]]! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'cwp 8/7/2003 19:18'!
recordVersionInfo
	Versions 
		at: self extractPackageName 
		put: self extractVersionInfo! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2003 22:58'!
installMember: member
	 self useNewChangeSetDuring:
		[member contentStream text fileInAnnouncing: 'loading ', member fileName]! !

!MczInstaller class methodsFor: 'services' stamp: 'cwp 8/7/2003 18:54'!
loadVersionFile: fileName
	self installFileNamed: fileName
! !

!MczInstaller class methodsFor: 'versionInfo' stamp: 'avi 1/19/2004 13:13'!
clearVersionInfo
	Versions _ Dictionary new! !

!MczInstaller class methodsFor: 'installing' stamp: 'cwp 8/7/2003 18:13'!
installFileNamed: aFileName
	self installStream: (FileStream readOnlyFileNamed: aFileName)! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'cwp 8/13/2003 11:19'!
extractInfoFrom: dict
	^ dict
		at: #id put: (UUID fromString: (dict at: #id));
		at: #date put: (Date fromString: (dict at: #date));
		at: #time put: (Time readFrom: (dict at: #time) readStream);
		at: #ancestors put: ((dict at: #ancestors) collect: [:ea | self extractInfoFrom: ea]);
		yourself
		! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'bf 2/9/2004 15:01'!
checkDependencies
	| dependencies unmet |
	dependencies _ (zip membersMatching: 'dependencies/*') 
			collect: [:member | self extractInfoFrom: (self parseMember: member)].
	unmet _ dependencies reject: [:dep |
		self versions: Versions anySatisfy: (dep at: #id)].
	^ unmet isEmpty or: [
		self confirm: (String streamContents: [:stream |
			stream nextPutAll: 'The following dependencies seem to be missing:'; cr.
			unmet do: [:each | stream nextPutAll: (each at: #name); cr].
			stream nextPutAll: 'Do you still want to install this package?'])]! !

!MczInstaller class methodsFor: 'instance creation' stamp: 'cwp 8/7/2003 17:56'!
on: aStream
	^ self new stream: aStream! !

!MczInstaller class methodsFor: 'versionInfo' stamp: 'cwp 8/7/2003 19:39'!
versionInfo
	^ Versions! !

!MczInstaller class methodsFor: 'services' stamp: 'cwp 8/7/2003 18:53'!
serviceLoadVersion
	^ SimpleServiceEntry
		provider: self
		label: 'load'
		selector: #loadVersionFile:
		description: 'load a package version'! !

!MczInstaller class methodsFor: 'services' stamp: 'cwp 8/7/2003 18:53'!
fileReaderServicesForFile: fileName suffix: suffix
	^ Array with: self serviceLoadVersion! !

!MczInstaller class methodsFor: 'installing' stamp: 'cwp 8/7/2003 17:56'!
installStream: aStream
	(self on: aStream) install! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'bf 2/9/2004 14:39'!
install
	| sources |
	self checkDependencies ifFalse: [^false].
	self recordVersionInfo.
	sources _ (zip membersMatching: 'snapshot/*') 
				asSortedCollection: [:a :b | a fileName < b fileName].
	sources do: [:src | self installMember: src].! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'cwp 8/13/2003 01:58'!
parseMember: fileName
	| tokens |
	tokens _ (self scanner scanTokens: (zip contentsOf: fileName)) first.
	^ self associate: tokens! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'cwp 8/13/2003 02:17'!
extractVersionInfo
	^ self extractInfoFrom: (self parseMember: 'version')! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'cwp 8/13/2003 01:58'!
extractPackageName
	^ (self parseMember: 'package') at: #name.
	! !

!MczInstaller class methodsFor: 'services' stamp: 'cwp 8/7/2003 18:49'!
extension
	^ 'mcz'! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'avi 1/20/2004 11:13'!
associate: tokens
	| result |
	result _ Dictionary new.
	tokens pairsDo: [:key :value | 
					value isString ifFalse: [value _ value collect: [:ea | self associate: ea]].
					value = 'nil' ifTrue: [value _ ''].
					result at: key put: value].
	^ result! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'cwp 8/13/2003 02:04'!
scanner
	^ Scanner new! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'bf 2/9/2004 13:56'!
useNewChangeSetDuring: aBlock
	| changeHolder oldChanges newChanges |
	changeHolder _ (ChangeSet respondsTo: #newChanges:)
						ifTrue: [ChangeSet]
						ifFalse: [Smalltalk].
	oldChanges _ (ChangeSet respondsTo: #current)
						ifTrue: [ChangeSet current]
						ifFalse: [Smalltalk changes].

	newChanges _ ChangeSet new name: (ChangeSet uniqueNameLike: self extractPackageName).
	changeHolder newChanges: newChanges.
	[aBlock value] ensure: [changeHolder newChanges: oldChanges].! !

!MczInstaller class methodsFor: 'services' stamp: 'ab 8/8/2003 18:01'!
services
	^ Array with: self serviceLoadVersion! !

!MczInstaller class methodsFor: 'services' stamp: 'ab 8/8/2003 18:02'!
initialize
	Versions _ Dictionary new.
	Smalltalk at: #MCReader ifAbsent: [FileList registerFileReader: self]! !

!MczInstaller methodsFor: 'as yet unclassified' stamp: 'cwp 8/7/2003 17:57'!
stream: aStream
	zip _ ZipArchive new.
	zip readFrom: aStream! !
MczInstaller initialize!
